#!/usr/bin/python3
#
# Copyright (C) 2019 Mario Limonciello <mario.limonciello@dell.com>
#
# SPDX-License-Identifier: LGPL-2.1+

import dbus
import os.path
import sys
import tempfile
import gi
gi.require_version('Fwupd', '2.0')
from gi.repository import Fwupd #pylint: disable=wrong-import-position
from simple_client import install, check_exists
from add_capsule_header import add_header
from firmware_packager import make_firmware_metainfo, create_firmware_cab

class Variables:
    def __init__(self, device_guid, version):
        self.device_guid = device_guid
        self.developer_name = "Dell Inc"
        self.firmware_name = "New firmware"
        self.firmware_summary = "Unknown"
        self.firmware_description = "Unknown"
        self.firmware_homepage = "https://support.dell.com"
        self.contact_info = "Unknown"
        self.release_version = version
        self.release_description = "Unknown"

def parse_args():
    """Parse arguments for this client"""
    import argparse
    parser = argparse.ArgumentParser(description="Interact with fwupd daemon")
    parser.add_argument('exe', nargs='?', help='exe file')
    parser.add_argument('deviceid', nargs='?',
                        help='DeviceID to operate on(optional)')
    args = parser.parse_args()
    return args

def generate_cab(infile, directory, guid, version):
    output = os.path.join(directory, "firmware.bin")
    ret = add_header(infile, output, guid)
    if ret:
        sys.exit(ret)
    variables = Variables(guid, version)
    make_firmware_metainfo(variables, directory)
    create_firmware_cab(variables, directory)
    cab = os.path.join(directory, "firmware.cab")
    print("Generated CAB file %s" % cab)
    return cab

def find_uefi_device(client, deviceid):
    devices = client.get_devices()
    for item in devices:
        #match the device we were given
        if deviceid:
            if item.get_id() != deviceid:
                continue
        # internal
        if not item.has_flag(1 << 0):
            continue
        # needs reboot
        if not item.has_flag(1 << 8):
            continue
        # return the first hit for UEFI plugin
        if item.get_plugin() == 'uefi':
            print("Installing to %s" % item.get_name())
            return item.get_guid_default(),item.get_id(),item.get_version()
    print("Couldn't find any UEFI devices")
    sys.exit(1)

def prompt_reboot():
    print("An update requires a reboot to complete")
    while True:
        res = input("Restart now? (Y/N) ")
        if res.lower() == 'n':
            print("Reboot your machine manually to finish the update.")
            break
        if res.lower() != 'y':
            continue
        #reboot using logind
        obj = dbus.SystemBus().get_object('org.freedesktop.login1',
                                          '/org/freedesktop/login1')
        obj.Reboot(True, dbus_interface='org.freedesktop.login1.Manager')

if __name__ == '__main__':
    ARGS = parse_args()
    CLIENT = Fwupd.Client()
    CLIENT.connect()
    check_exists(ARGS.exe)
    directory = tempfile.mkdtemp()
    guid, deviceid, version=find_uefi_device(CLIENT, ARGS.deviceid)
    cab = generate_cab(ARGS.exe, directory, guid, version)
    install(CLIENT, cab, deviceid, True, True)
    prompt_reboot()


#!/usr/bin/python3
# SPDX-License-Identifier: LGPL-2.1+

import argparse
import sys
import subprocess
import os

def main():
    parser = argparse.ArgumentParser(description='Run afl-fuzz on all cores')
    parser.add_argument('--input', '-i', help='fuzzing input directory')
    parser.add_argument('--output', '-o', help='findings output directory')
    parser.add_argument('--command', type=str, help='fuzzer tool command')
    parser.add_argument('path', type=str, help='the fuzzer tool')
    args = parser.parse_args()
    if not args.input and not args.output:
        print('-i and -o required')
        return 1
    if not args.path:
        print('tool name required')
        return 1

    # create if not already exists
    if not os.path.exists(args.output):
        os.makedirs(args.output)

    # run the main instance
    envp = None
    argv = ['afl-fuzz', '-m300', '-i', args.input, '-o', args.output,
            '-M', 'fuzzer00', args.path]
    if args.command:
        argv.append(args.command)
    argv.append('@@')
    print(argv)
    p = subprocess.Popen(argv, env=envp)

    # run the secondary instances
    cs = []
    for i in range(1, os.cpu_count()):
        argv = ['afl-fuzz', '-m300', '-i', args.input, '-o', args.output,
                '-S', 'fuzzer%02i' % i, args.path]
        if args.command:
            argv.append(args.command)
        argv.append('@@')
        print(argv)
        cs.append(subprocess.Popen(argv, env=envp, stdout=subprocess.DEVNULL))

    # wait for the main instance
    try:
        p.wait()
    except KeyboardInterrupt as _:
        pass
    for c in cs:
        c.terminate()
    return 0

if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/python3
#
# Copyright (C) 2017 Max Ehrlich max.ehr@gmail.com
#
# SPDX-License-Identifier: LGPL-2.1+
#

import argparse
import subprocess
import contextlib
import os
import shutil
import tempfile
import time


@contextlib.contextmanager
def cd(path):
    prev_cwd = os.getcwd()
    os.chdir(path)
    yield
    os.chdir(prev_cwd)

firmware_metainfo_template = """
<?xml version="1.0" encoding="UTF-8"?>
<component type="firmware">
  <id>org.{developer_name}.guid{firmware_id}</id>
  <name>{firmware_name}</name>
  <summary>{firmware_summary}</summary>
  <description>
    {firmware_description}
  </description>
  <provides>
    <firmware type="flashed">{device_guid}</firmware>
  </provides>
  <url type="homepage">{firmware_homepage}</url>
  <metadata_license>CC0-1.0</metadata_license>
  <project_license>proprietary</project_license>
  <updatecontact>{contact_info}</updatecontact>
  <developer_name>{developer_name}</developer_name>
  <releases>
    <release version="{release_version}" timestamp="{timestamp}">
      <description>
        {release_description}
      </description>
    </release>
  </releases>
</component>
"""


def make_firmware_metainfo(firmware_info, dst):
    local_info = vars(firmware_info)
    local_info["firmware_id"] = local_info["device_guid"][0:8]
    firmware_metainfo = firmware_metainfo_template.format(**local_info, timestamp=time.time())

    with open(os.path.join(dst, 'firmware.metainfo.xml'), 'w') as f:
        f.write(firmware_metainfo)


def extract_exe(exe, dst):
    command = ['7z', 'x', '-o{}'.format(dst), exe]
    subprocess.check_call(command, stdout=subprocess.DEVNULL)


def get_firmware_bin(root, bin_path, dst):
    with cd(root):
        shutil.copy(bin_path, os.path.join(dst, 'firmware.bin'))


def create_firmware_cab(exe, folder):
    with cd(folder):
        if os.name == "nt":
          directive = os.path.join (folder, "directive")
          with open (directive, 'w') as wfd:
            wfd.write('.OPTION EXPLICIT\r\n')
            wfd.write('.Set CabinetNameTemplate=firmware.cab\r\n')
            wfd.write('.Set DiskDirectory1=.\r\n')
            wfd.write('firmware.bin\r\n')
            wfd.write('firmware.metainfo.xml\r\n')
          command = ['makecab.exe', '/f', directive]
        else:
          command = ['gcab', '--create', 'firmware.cab', 'firmware.bin', 'firmware.metainfo.xml']
        subprocess.check_call(command)


def main(args):
    with tempfile.TemporaryDirectory() as dir:
        print('Using temp directory {}'.format(dir))

        if args.exe:
            print('Extracting firmware exe')
            extract_exe(args.exe, dir)

        print('Locating firmware bin')
        get_firmware_bin(dir, args.bin, dir)

        print('Creating metainfo')
        make_firmware_metainfo(args, dir)

        print('Cabbing firmware files')
        create_firmware_cab(args, dir)

        print('Done')
        shutil.copy(os.path.join(dir, 'firmware.cab'), args.out)

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Create fwupd packaged from windows executables')
    parser.add_argument('--firmware-name', help='Name of the firmware package can be customized (e.g. DellTBT)', required=True)
    parser.add_argument('--firmware-summary', help='One line description of the firmware package')
    parser.add_argument('--firmware-description', help='Longer description of the firmware package')
    parser.add_argument('--device-guid', help='GUID of the device this firmware will run on, this *must* match the output of one of the GUIDs in `fwupdmgr get-devices`', required=True)
    parser.add_argument('--firmware-homepage', help='Website for the firmware provider')
    parser.add_argument('--contact-info', help='Email address of the firmware developer')
    parser.add_argument('--developer-name', help='Name of the firmware developer', required=True)
    parser.add_argument('--release-version', help='Version number of the firmware package', required=True)
    parser.add_argument('--release-description', help='Description of the firmware release')
    parser.add_argument('--exe', help='(optional) Executable file to extract firmware from')
    parser.add_argument('--bin', help='Path to the .bin file (Relative if inside the executable; Absolute if outside) to use as the firmware image', required=True)
    parser.add_argument('--out', help='Output cab file path', required=True)
    args = parser.parse_args()

    main(args)



#!/usr/bin/python3
#
# Copyright (C) 2019 Dell, Inc.
#
# SPDX-License-Identifier: LGPL-2.1+
#

import xml.etree.ElementTree as etree
import os
import subprocess

def sanitize_for_ci(version):
    if not 'CI' in os.environ:
        return version
    OS=os.getenv('OS')
    if not OS:
        return version
    if "fedora" in OS:
        return version.replace('-','.')
    return version

def get_version_git():
    try:
        version = subprocess.check_output(['git', 'describe'], stderr=subprocess.DEVNULL)
        return version.strip().decode('utf-8')
    except (subprocess.CalledProcessError, PermissionError, FileNotFoundError):
        return ''

def get_version():
    tree = etree.parse(os.path.join("data", "org.freedesktop.fwupd.metainfo.xml"))
    version = ''
    for child in tree.findall('releases'):
        for release in child:
            if not "version" in release.attrib:
                continue
            if release.attrib['version'] > version:
                version = release.attrib['version']
    return version

if __name__ == '__main__':

    version = get_version_git()
    if version:
        version = sanitize_for_ci(version)
    else:
        version = get_version()
    print(version)
